`timescale 1ns / 1ps

module fir_filter(
      input clk,
      input rst,
      
      input [1:0] din_valid,
      input [23:0] din,
      
      output [1:0] dout_valid,
      output [23:0] dout
);

// vezrl llapot:
//  0: nincs szrs
//  1: aktv szrs
reg state;
// egytthat cmszmll
//  255...0 kztt szmol, konvolci kezdetekor 255-re lltjuk
reg [7:0] coeff_addr_reg;



// feldolgozs alatt ll csatorna
// konvolci kezdetekor elmentjk, hogy melyik bemenet volt rvnyes
reg ch_act;



// aktv szrs (state) ksleltetse
reg [7:0] state_1_dl;


// egytthat ROM cm:
//   {aktv csatorna, egytthat cmszmll}
wire [8:0] coeff_addr;


// minta rsi cmszmll
//   din_valid[1]-re inkrementl
reg [7:0] smpl_wr_addr_reg;



// minta rsi cm
//   {input valid, cmszmll}
wire [8:0] smpl_wr_addr;


// olvassi cmszmll
// smpl_wr_addr_reg_rl indul j minta rkezsekor, dekrementldik
reg [7:0] smpl_rd_addr_reg;



// olvassi cm: {aktv csatorna, cmszmll}
wire [8:0] smpl_rd_addr;



// Mintatr (24 bit szles)
// bementi mintk: s.23
wire [23:0] smpl_ram_dout;
ram #(
   .DATA_W(24),
   .ADDR_W(9)
)
smpl_ram(
   .clk_a  (clk),
   .we_a   ((din_valid!=0)),
   .addr_a (smpl_wr_addr),
   .din_a  (din),
   .dout_a (),
   .clk_b  (clk),
   .we_b   (1'b0),
   .addr_b (smpl_rd_addr),
   .din_b  (36'b0),
   .dout_b (smpl_ram_dout)
);

// Egytthat ROM
// egytthatk: s.3.31
wire [34:0] coeff_rom_dout;
rom_512x35 coeff_rom(
   .clk  (clk),
   .addr (coeff_addr),
   .dout (coeff_rom_dout)
);

// rszszorzat rdekes rsze:
// minta: 24 bit
// coeff: 35 bit
//    azaz:  59 bit
//  s.23*s.3.31 = s.4.54
wire signed [58:0] mul_res;
mul_24x35 mul_fir(
      .clk  (clk),
      .a    (smpl_ram_dout),
      .b    (coeff_rom_dout),
      .m    (mul_res)
);

// Accu reset: az els? rvnyes bemenet alatt
// Engedlyezs: amikor rvnyes a bemenete (state[1] ksleltetve pipeline latency-vel)
wire accu_rst;
wire accu_en;



// Reset: az rbnyes bemenetet rjuk be akkumulls nlkl
// 256 db s.4.54 sszege --> s.12.54 --> 67 bit
reg signed [66:0] accu;




// kimeneti formtum: s.23,
//  accu eredmny-bl levgjuk az als 31 bitet, az ezt kvet 24 bit az rvnyes kimenet
//  kivve ha tlcsorduls van, ekkor szaturci:
//    - pozitv: +0.999999 --> h7fffff
//    - negatv: -1        --> h800000

//  Kimenet rvnyes: csatorna + accu_en lefut l
reg [23:0] dout_reg;
reg  [1:0] dout_valid_reg;




assign dout = dout_reg;
assign dout_valid = dout_valid_reg;

   
endmodule
